"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const chokidar_1 = __importDefault(require("chokidar"));
const parseSFC_1 = require("./parseSFC");
function createFileWatcher(notify) {
    const fileWatcher = chokidar_1.default.watch(process.cwd(), {
        ignored: [/node_modules/]
    });
    fileWatcher.on('change', async (file) => {
        const resourcePath = '/' + path_1.default.relative(process.cwd(), file);
        if (file.endsWith('.vue')) {
            // check which part of the file changed
            const [descriptor, prevDescriptor] = await parseSFC_1.parseSFC(file);
            if (!prevDescriptor) {
                // the file has never been accessed yet
                return;
            }
            if ((descriptor.script && descriptor.script.content) !==
                (prevDescriptor.script && prevDescriptor.script.content)) {
                console.log(`[hmr:reload] ${resourcePath}`);
                notify({
                    type: 'reload',
                    path: resourcePath
                });
                return;
            }
            if ((descriptor.template && descriptor.template.content) !==
                (prevDescriptor.template && prevDescriptor.template.content)) {
                console.log(`[hmr:rerender] ${resourcePath}`);
                notify({
                    type: 'rerender',
                    path: resourcePath
                });
                return;
            }
            // TODO styles
        }
        else {
            console.log(`[hmr:full-reload] ${resourcePath}`);
            notify({
                type: 'full-reload'
            });
        }
    });
}
exports.createFileWatcher = createFileWatcher;
