"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const resolve_from_1 = __importDefault(require("resolve-from"));
const utils_1 = require("./utils");
const fileToIdMap = new Map();
const resolveCache = new Map();
const sendWithCache = (res, id, modulePath) => {
    resolveCache.set(id, modulePath);
    utils_1.sendJSStream(res, modulePath);
};
function resolveVue(cwd) {
    if (resolveCache.has('vue')) {
        return resolveCache.get('vue');
    }
    let vuePath;
    let compilerPath;
    try {
        // see if user has local vue installation
        const userVuePkg = resolve_from_1.default(cwd, 'vue/package.json');
        vuePath = path_1.default.join(path_1.default.dirname(userVuePkg), 'dist/vue.runtime.esm-browser.js');
        // also resolve matching sfc
        try {
            const compilerPkgPath = resolve_from_1.default(cwd, '@vue/compiler-sfc/package.json');
            const compilerPkg = require(compilerPkgPath);
            if (compilerPkg.version !== require(userVuePkg).version) {
                throw new Error();
            }
            compilerPath = path_1.default.join(path_1.default.dirname(compilerPkgPath), compilerPkg.main);
        }
        catch (e) {
            // user has local vue but has no compiler-sfc
            console.error(`[vite] Error: a local installation of \`vue\` is detected but ` +
                `no matching \`@vue/compiler-sfc\` is found. Make sure to install ` +
                `both and use the same version.`);
            compilerPath = require.resolve('@vue/compiler-sfc');
        }
    }
    catch (e) {
        // user has no local vue, use vite's dependency version
        vuePath = require.resolve('vue/dist/vue.runtime.esm-browser.js');
        compilerPath = require.resolve('@vue/compiler-sfc');
    }
    resolveCache.set('vue', vuePath);
    resolveCache.set('@vue/compiler-sfc', compilerPath);
    return vuePath;
}
exports.resolveVue = resolveVue;
function resolveCompiler(cwd) {
    resolveVue(cwd);
    return require(resolveCache.get('@vue/compiler-sfc'));
}
exports.resolveCompiler = resolveCompiler;
// TODO support custom imports map e.g. for snowpack web_modules
function resolveModule(id, cwd, res) {
    if (id === 'vue') {
        // special handling for vue
        return utils_1.sendJSStream(res, resolveVue(cwd));
    }
    // already cached
    let modulePath = resolveCache.get(id);
    if (modulePath) {
        return utils_1.sendJSStream(res, modulePath);
    }
    // handle source map requests
    let sourceMapPath = undefined;
    if (id.endsWith('.map')) {
        sourceMapPath = id;
        id = fileToIdMap.get(id.replace(/\.map$/, ''));
        if (!id) {
            res.statusCode = 404;
            res.end();
            return;
        }
    }
    // fallback to node resolve
    try {
        modulePath = resolve_from_1.default(cwd, `${id}/package.json`);
        // module resolved, try to locate its "module" entry
        const pkg = require(modulePath);
        modulePath = path_1.default.join(path_1.default.dirname(modulePath), pkg.module || pkg.main);
        fileToIdMap.set(path_1.default.basename(modulePath), id);
        // this is a source map request.
        if (sourceMapPath) {
            modulePath = path_1.default.join(path_1.default.dirname(modulePath), sourceMapPath);
            return sendWithCache(res, sourceMapPath, modulePath);
        }
        sendWithCache(res, id, modulePath);
    }
    catch (e) {
        console.error(e);
        res.statusCode = 404;
        res.end();
    }
}
exports.resolveModule = resolveModule;
