"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const parser_1 = require("@babel/parser");
const magic_string_1 = __importDefault(require("magic-string"));
function rewrite(source, asSFCScript = false) {
    const ast = parser_1.parse(source, {
        sourceType: 'module',
        plugins: [
            // by default we enable proposals slated for ES2020.
            // full list at https://babeljs.io/docs/en/next/babel-parser#plugins
            // this will need to be updated as the spec moves forward.
            'bigInt',
            'optionalChaining',
            'nullishCoalescingOperator'
        ]
    }).program.body;
    const s = new magic_string_1.default(source);
    ast.forEach((node) => {
        if (node.type === 'ImportDeclaration') {
            if (/^[^\.\/]/.test(node.source.value)) {
                // module import
                // import { foo } from 'vue' --> import { foo } from '/__modules/vue'
                s.overwrite(node.source.start, node.source.end, `"/__modules/${node.source.value}"`);
            }
        }
        else if (asSFCScript && node.type === 'ExportDefaultDeclaration') {
            s.overwrite(node.start, node.declaration.start, `let __script; export default (__script = `);
            s.appendRight(node.end, `)`);
        }
    });
    return s.toString();
}
exports.rewrite = rewrite;
