"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const url_1 = __importDefault(require("url"));
const path_1 = __importDefault(require("path"));
const fs_1 = require("fs");
const utils_1 = require("./utils");
const moduleRewriter_1 = require("./moduleRewriter");
const moduleResolver_1 = require("./moduleResolver");
const hash_sum_1 = __importDefault(require("hash-sum"));
const cache = new Map();
async function parseSFC(cwd, filename, saveCache = false) {
    let content;
    try {
        content = await fs_1.promises.readFile(filename, 'utf-8');
    }
    catch (e) {
        return [];
    }
    const { descriptor, errors } = moduleResolver_1.resolveCompiler(cwd).parse(content, {
        filename
    });
    if (errors) {
        // TODO
    }
    const prev = cache.get(filename);
    if (saveCache) {
        cache.set(filename, descriptor);
    }
    return [descriptor, prev];
}
exports.parseSFC = parseSFC;
async function vueMiddleware(cwd, req, res) {
    const parsed = url_1.default.parse(req.url, true);
    const pathname = parsed.pathname;
    const query = parsed.query;
    const filename = path_1.default.join(cwd, pathname.slice(1));
    const [descriptor] = await parseSFC(cwd, filename, true /* save last accessed descriptor on the client */);
    if (!descriptor) {
        res.statusCode = 404;
        return res.end();
    }
    if (!query.type) {
        return compileSFCMain(res, descriptor, pathname, query.t);
    }
    if (query.type === 'template') {
        return compileSFCTemplate(cwd, res, descriptor.template, filename, pathname, descriptor.styles.some((s) => s.scoped));
    }
    if (query.type === 'style') {
        return compileSFCStyle(cwd, res, descriptor.styles[Number(query.index)], query.index, filename, pathname);
    }
    // TODO custom blocks
}
exports.vueMiddleware = vueMiddleware;
function compileSFCMain(res, descriptor, pathname, timestamp) {
    timestamp = timestamp ? `&t=${timestamp}` : ``;
    // inject hmr client
    let code = `import "/__hmrClient"\n`;
    if (descriptor.script) {
        code += moduleRewriter_1.rewrite(descriptor.script.content, true /* rewrite default export to `script` */);
    }
    else {
        code += `const __script = {}; export default __script`;
    }
    let hasScoped = false;
    if (descriptor.styles) {
        descriptor.styles.forEach((s, i) => {
            if (s.scoped)
                hasScoped = true;
            code += `\nimport ${JSON.stringify(pathname + `?type=style&index=${i}${timestamp}`)}`;
        });
        if (hasScoped) {
            code += `\n__script.__scopeId = "data-v-${hash_sum_1.default(pathname)}"`;
        }
    }
    if (descriptor.template) {
        code += `\nimport { render as __render } from ${JSON.stringify(pathname + `?type=template${timestamp}`)}`;
        code += `\n__script.render = __render`;
    }
    code += `\n__script.__hmrId = ${JSON.stringify(pathname)}`;
    utils_1.sendJS(res, code);
}
function compileSFCTemplate(cwd, res, template, filename, pathname, scoped) {
    const { code, errors } = moduleResolver_1.resolveCompiler(cwd).compileTemplate({
        source: template.content,
        filename,
        compilerOptions: {
            scopeId: scoped ? `data-v-${hash_sum_1.default(pathname)}` : null,
            runtimeModuleName: '/__modules/vue'
        }
    });
    if (errors) {
        // TODO
    }
    utils_1.sendJS(res, code);
}
function compileSFCStyle(cwd, res, style, index, filename, pathname) {
    const id = hash_sum_1.default(pathname);
    const { code, errors } = moduleResolver_1.resolveCompiler(cwd).compileStyle({
        source: style.content,
        filename,
        id: `data-v-${id}`,
        scoped: style.scoped != null
    });
    // TODO css modules
    if (errors) {
        // TODO
    }
    utils_1.sendJS(res, `
const id = "vue-style-${id}-${index}"
let style = document.getElementById(id)
if (!style) {
  style = document.createElement('style')
  style.id = id
  document.head.appendChild(style)
}
style.textContent = ${JSON.stringify(code)}
  `.trim());
}
