"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const url_1 = __importDefault(require("url"));
const path_1 = __importDefault(require("path"));
const parseSFC_1 = require("./parseSFC");
const compiler_sfc_1 = require("@vue/compiler-sfc");
const utils_1 = require("./utils");
const moduleRewriter_1 = require("./moduleRewriter");
async function vueMiddleware(req, res) {
    const parsed = url_1.default.parse(req.url, true);
    const query = parsed.query;
    const filename = path_1.default.join(process.cwd(), parsed.pathname.slice(1));
    const [descriptor] = await parseSFC_1.parseSFC(filename, true /* save last accessed descriptor on the client */);
    if (!query.type) {
        // inject hmr client
        let code = `import "/__hmrClient"\n`;
        if (descriptor.script) {
            code += moduleRewriter_1.rewrite(descriptor.script.content, true /* rewrite default export to `script` */);
        }
        else {
            code += `const __script = {}; export default __script`;
        }
        if (descriptor.template) {
            code += `\nimport { render as __render } from ${JSON.stringify(parsed.pathname + `?type=template${query.t ? `&t=${query.t}` : ``}`)}`;
            code += `\n__script.render = __render`;
        }
        if (descriptor.style) {
            // TODO
        }
        code += `\n__script.__hmrId = ${JSON.stringify(parsed.pathname)}`;
        return utils_1.sendJS(res, code);
    }
    if (query.type === 'template') {
        const { code, errors } = compiler_sfc_1.compileTemplate({
            source: descriptor.template.content,
            filename,
            compilerOptions: {
                // TODO infer proper Vue path
                runtimeModuleName: '/__modules/vue'
            }
        });
        if (errors) {
            // TODO
        }
        return utils_1.sendJS(res, code);
    }
    if (query.type === 'style') {
        // TODO
        return;
    }
    // TODO custom blocks
}
exports.vueMiddleware = vueMiddleware;
