"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const chokidar_1 = __importDefault(require("chokidar"));
const vueCompiler_1 = require("./vueCompiler");
const hash_sum_1 = __importDefault(require("hash-sum"));
function createFileWatcher(cwd, notify) {
    const fileWatcher = chokidar_1.default.watch(cwd, {
        ignored: [/node_modules/]
    });
    fileWatcher.on('change', async (file) => {
        const resourcePath = '/' + path_1.default.relative(cwd, file);
        const send = (payload) => {
            console.log(`[hmr] ${JSON.stringify(payload)}`);
            notify(payload);
        };
        if (file.endsWith('.vue')) {
            // check which part of the file changed
            const [descriptor, prevDescriptor] = await vueCompiler_1.parseSFC(cwd, file);
            if (!descriptor || !prevDescriptor) {
                // the file has never been accessed yet
                return;
            }
            if (!isEqual(descriptor.script, prevDescriptor.script)) {
                send({
                    type: 'reload',
                    path: resourcePath
                });
                return;
            }
            if (!isEqual(descriptor.template, prevDescriptor.template)) {
                send({
                    type: 'rerender',
                    path: resourcePath
                });
                return;
            }
            const prevStyles = prevDescriptor.styles || [];
            const nextStyles = descriptor.styles || [];
            if (prevStyles.some((s) => s.scoped) !== nextStyles.some((s) => s.scoped)) {
                send({
                    type: 'reload',
                    path: resourcePath
                });
            }
            nextStyles.forEach((_, i) => {
                if (!prevStyles[i] || !isEqual(prevStyles[i], nextStyles[i])) {
                    send({
                        type: 'style-update',
                        path: resourcePath,
                        index: i
                    });
                }
            });
            prevStyles.slice(nextStyles.length).forEach((_, i) => {
                send({
                    type: 'style-remove',
                    path: resourcePath,
                    id: `${hash_sum_1.default(resourcePath)}-${i + nextStyles.length}`
                });
            });
        }
        else {
            send({
                type: 'full-reload'
            });
        }
    });
}
exports.createFileWatcher = createFileWatcher;
function isEqual(a, b) {
    if (!a || !b)
        return false;
    if (a.content !== b.content)
        return false;
    const keysA = Object.keys(a.attrs);
    const keysB = Object.keys(b.attrs);
    if (keysA.length !== keysB.length) {
        return false;
    }
    return keysA.every((key) => a.attrs[key] === b.attrs[key]);
}
